Unit MomokoCodeTrans;
Interface

Uses
 Windows, Messages, SysUtils, Classes, Graphics, MMSystem, TypInfo,
 Controls, Forms, Dialogs, ExtCtrls, ShellAPI, StrUtils;

Const
 MainVersion = 1;
 BuildVersion = 1772;

Type

 TTransMode = (tmNoTrans, tmShiftJIS, tmBIG5, tmGB18030);

 TCountry = (
             Arabic, Chinese, English, Saudi_Arabia, Bulgarian, Catalan, Chinese_Taiwan, Czech, Danish,
             German, Greek, United_States, Traditional_Sort, Finnish, French, Hebrew, Hungarian, Icelandic,
             Italian, Japanese, Korean, Dutch, Norwegian, Polish, Brazil, Rhaeto_Romanic, Romanian, Russian,
             Croatian, Slovak, Albanian, Swedish, Thai, Turkish, Urdu, Indonesian, Ukrainian, Belarusian, Slovenian,
             Estonian, Latvian, Lithuanian, Farsi, Vietnamese, Basque, Serbian, FYROM, Sutu, Tsonga, Tswana, Xhosa, Zulu,
             Afrikaans, Faeroese, Hindi, Maltese, Gaelic, Yiddish, Malaysia, Iraq, Chinese_PRC, Switzerland, United_Kingdom,
             Spanish_Mexico, French_Belgium, Italian_Switzerland, Dutch_Belgium, Norwegian_Nynorsk, Portuguese_Portugal, Romanian_Moldova,
             Russian_Moldova, Serbian_Latin, Swedish_Finland, Arabic_Egypt, Chinese_Hong_Kong_SAR, German_Austria, English_Australia, Spanish_International_Sort,
             French_Canada, Serbian_Cyrillic, Arabic_Libya, Chinese_Singapore, German_Luxembourg, English_Canada, Spanish_Guatemala, French_Switzerland, Arabic_Algeria,
             German_Liechtenstein, English_New_Zealand, Spanish_Costa_Rica, French_Luxembourg, Arabic_Morocco, English_Ireland, Spanish_Panama, Arabic_Tunisia, English_South_Africa,
             Spanish_Dominican_Republic, Arabic_Oman, English_Jamaica, Spanish_Venezuela, Arabic_Yemen, Spanish_Colombia, Arabic_Syria, English_Belize, Spanish_Peru, Arabic_Jordan, English_Trinidad,
             Spanish_Argentina, Arabic_Lebanon, Spanish_Ecuador, Arabic_Kuwait, Spanish_Chile, Arabic_U_A_E, Spanish_Uruguay, Arabic_Bahrain, Spanish_Paraguay, Arabic_Qatar, Spanish_Bolivia, Spanish_El_Salvador,
             Spanish_Honduras, Spanish_Nicaragua, Spanish_Puerto_Rico, Unknown
             );

 TMomoko = Class(TCustomControl)
 Private
  FPicture: TBitmap;
  FDevices: Integer;
  FTransMode: TTransMode;
  FDummy: Integer;
  Function GetVolume:Byte;
  Procedure SetVolume(aVolume: Byte);
  Procedure SetTransMode(Mode: TTransMode);
  Procedure InitVolume;
  Procedure WMPaint(Var Message: TWMPaint); Message WM_PAINT;
  Procedure WMSize(Var Message: TWMSize); Message WM_SIZE;
 Protected
  Function GetCountry:TCountry;
  Procedure SetCountry(Value: TCountry);
  Procedure Notification(AComponent: TComponent; AOperation: TOperation);Override;
 Public
  Constructor Create(AOwner: TComponent); Override;
  Destructor Destroy; Override;
 Published
  Property Cursor: Integer Read FDummy;
  Property Tag: Integer Read FDummy;
  Property HelpContext: Integer Read FDummy;
  Property HelpKeyword: Integer Read FDummy;
  Property HelpType: Integer Read FDummy;
  Property Hint: Integer Read FDummy;
  Property TranslateMode:TTransMode Read FTransMode Write SetTransMode Default tmNoTrans;
  Property Volume:Byte Read GetVolume Write SetVolume Stored False;
  Property Country:TCountry Read GetCountry Write SetCountry Stored False;
  Function Translate(SentencesOfGB2312: String):String;
  Procedure GetLanguageSettings(Country: TCountry; Var CharSet: TFontCharSet; Var FontName: TFontName);
  Procedure UseLanguageSettings(AimObj: TComponent; Country: TCountry);
  Procedure SetFontSizeTo9pixels(AimObj: TComponent);
  Procedure PrepareLocalizeSource(WindowNameForLocalize: TForm; UnitName, SourceForLocalize: String);
 End;

Procedure Register;

Implementation

{$R 'MomokoCodeTrans.dcr'}

Uses
 CodeTrans;

Procedure Register;
 Begin
  RegisterComponents('Floatsoft',[TMomoko]);
 End;

Type

 TVolumeRec = Record
               Case Integer Of
               0: (LongVolume: Longint);
               1: (LeftVolume, RightVolume : Word);
               End;

Procedure TMomoko.SetFontSizeTo9pixels(AimObj: TComponent);
 Var
  i,j: Integer;
  o: TComponent;
  f: TFont;
  p: PPropInfo;
 Begin
  o := AimObj;
  p := GetPropInfo(o.ClassInfo, 'Font');
  If p <> Nil Then
   If p^.PropType^.Name = 'TFont' Then
    Begin
     f := TFont(GetOrdProp(o, p));
     f.Size := 9;
    End;
  j := AimObj.ComponentCount - 1;
  For i := 0 to j Do
   Begin
    o := AimObj.Components[i];
    SetFontSizeTo9pixels(o);
   End;
 End;

Procedure TMomoko.PrepareLocalizeSource(WindowNameForLocalize: TForm; UnitName, SourceForLocalize: String);
 Var
  SaveDialog: TSaveDialog;
  t: TextFile;
  u: String;
  i: Integer;
  f: TForm;
  Procedure WriteResource(AimObj: TComponent; Country: TCountry; PathName: String);
   Var
    i,j: Integer;
    o: TComponent;
    n: TFontName;
    c: TFontCharset;
    p: PPropInfo;
    f: TCaption;
    s, u: String;
   Begin
    GetLanguageSettings(Country, c, n);
    o := AimObj;
    If PathName = '' Then s := o.Name Else s:= PathName + '.' + o.Name;
    p := GetPropInfo(o.ClassInfo, 'Font');
    If p <> Nil Then
     If p^.PropType^.Name = 'TFont' Then
      Begin
       WriteLn(t, '     ' + s + o.Name + '.Font.Charset := ' + IntToStr(c) + ';');
       WriteLn(t, '     ' + s + o.Name + '.Font.Name := ''' + n + ''';');
      End;
    p := GetPropInfo(o.ClassInfo, 'Caption');
    If p <> Nil Then
     If p^.PropType^.Name = 'TCaption' Then
      Begin
       f := GetStrProp(o, p);
       u := f;
       Case Country Of
       Japanese: u := TranslateGBK2JISbyString(u);
       Chinese_Taiwan: u := TranslateGBK2BIG5byString(u);
       End;
       WriteLn(t, '     ' + s + o.Name + '.Caption := ''' + u + ''';');
      End;
    p := GetPropInfo(o.ClassInfo, 'Text');
    If p <> Nil Then
     If p^.PropType^.Name = 'TCaption' Then
      Begin
       f := GetStrProp(o, p);
       u := f;
       Case Country Of
       Japanese: u := TranslateGBK2JISbyString(u);
       Chinese_Taiwan: u := TranslateGBK2BIG5byString(u);
       End;
       WriteLn(t, '     ' + s + o.Name + '.Text := ''' + u + ''';');
      End;
    j := AimObj.ComponentCount - 1;
    For i := 0 to j Do
     Begin
      If PathName = '' Then s := o.Name Else s := PathName + '.' + o.Name;
      o := AimObj.Components[i];
      WriteResource(o, Country, s);
     End;
   End;
 Begin
  If UnitName = '' Then
   Begin
    MessageBox(Handle, 'The aim Source filename for inserting has not input yet.','Momoko:',MB_OK);
    Exit;
   End;
  If SourceForLocalize = '' Then
   Begin
    SaveDialog := TSaveDialog.Create(Self);
    SaveDialog.Title := 'Save a Source ready for Localization:';
    SaveDialog.Filter := 'Delphi Unit(*.pas)|*.pas';
    SaveDialog.Filename := 'LocalizationFor' + WindowNameForLocalize.Name + '.Pas';
    If SaveDialog.Execute Then SourceForLocalize := SaveDialog.FileName;
    SaveDialog.Free;
    If SourceForLocalize = '' Then
     Begin
      MessageBox(Handle, 'The Source filename has not input yet.','Momoko:',MB_OK);
      Exit;
     End;
   End;
  f := WindowNameForLocalize;
  If f = Nil Then
   Begin
    MessageBox(Handle, 'The Form which your specified cannot be found.','Momoko:',MB_OK);
    Exit;
   End;
  AssignFile(t, SourceForLocalize);
  Try
   ReWrite(t);
   u := ExtractFileName(SourceForLocalize);
   i := Pos('.', u);
   If i > 0 Then u := LeftStr(u, i - 1);
   WriteLn(t, 'Unit ' + u + ';');
   WriteLn(t);
   WriteLn(t, ' Interface');
   WriteLn(t);
   WriteLn(t, ' Uses');
   WriteLn(t, '  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, ExtCtrls;');
   WriteLn(t);
   WriteLn(t, ' Type');
   WriteLn(t, '  Language = (ShiftJIS0812, GB2312, BIG5);');
   WriteLn(t);
   WriteLn(t, ' Procedure LocalizeIt(AimLanguage: Language);');
   WriteLn(t);
   WriteLn(t, 'Implementation');
   WriteLn(t);
   WriteLn(t, ' Uses');
   WriteLn(t, '  ' + UnitName + ';');
   WriteLn(t);
   WriteLn(t, ' Procedure LocalizeIt(AimLanguage: Language);');
   WriteLn(t, '  Begin');
   WriteLn(t, '   Case AimLanguage Of');
   WriteLn(t, '   ShiftJIS0812:');
   WriteLn(t, '    Begin');
   WriteResource(f, Japanese, f.Name);
   WriteLn(t, '     // Here write your other code for Shift-Jis-0812: for example, TTreeView item must translate and so do the TMemo.');
   WriteLn(t);
   WriteLn(t);
   WriteLn(t, '     // Shift-Jis-0812 code end.');
   WriteLn(t, '    End;');
   WriteLn(t, '   BIG5:');
   WriteLn(t, '    Begin');
   WriteResource(f, Chinese_Taiwan, f.Name);
   WriteLn(t, '     // Here write your other code for Shift-Jis-0812: for example, TTreeView item must translate and so do the TMemo.');
   WriteLn(t);
   WriteLn(t);
   WriteLn(t, '     // Shift-Jis-0812 code end.');
   WriteLn(t, '    End;');
   WriteLn(t, '   GB2312:');
   WriteLn(t, '    Begin');
   WriteResource(f, Chinese_PRC, f.Name);
   WriteLn(t, '     // Here write your other code for Shift-Jis-0812: for example, TTreeView item must translate and so do the TMemo.');
   WriteLn(t);
   WriteLn(t);
   WriteLn(t, '     // Shift-Jis-0812 code end.');
   WriteLn(t, '    End;');   
   WriteLn(t, '   End;');
   WriteLn(t, '  End;');
   WriteLn(t, 'End.');
   MessageBox(Handle, 'The Source file has successfully created, please include it and append more functions.','Momoko:',MB_OK);
  Except
   MessageBox(Handle, 'Met a disk error when output source to the file your specified.','Momoko:',MB_OK);
  End;
  CloseFile(t);
 End;

Procedure TMomoko.UseLanguageSettings(AimObj: TComponent; Country: TCountry);
 Var
  i,j: Integer;
  o: TComponent;
  n: TFontName;
  c: TFontCharset;
  p: PPropInfo;
  f: TFont;
 Begin
  GetLanguageSettings(Country, c, n);
  o := AimObj;
  p := GetPropInfo(o.ClassInfo, 'Font');
  If p <> Nil Then
   If p^.PropType^.Name = 'TFont' Then
    Begin
     f := TFont(GetOrdProp(o, p));
     f.Charset := c;
     f.Name := n;
    End;
  j := AimObj.ComponentCount - 1;
  For i := 0 to j Do
   Begin
    o := AimObj.Components[i];
    UseLanguageSettings(o, Country);
   End;
 End;

Procedure TMomoko.GetLanguageSettings(Country: TCountry; Var CharSet: TFontCharSet; Var FontName: TFontName);
 Begin
  CharSet := GB2312_CHARSET;
  FontName := 'SimSun';
  Case Country Of
  Chinese_Taiwan: Begin
                   CharSet := CHINESEBIG5_CHARSET;
                   FontName := 'PMingLiU';
                  End;
  Japanese: Begin
             CharSet := SHIFTJIS_CHARSET;
             FontName := 'MS UI Gothic';
            End;
  Korean: Begin
           CharSet := HANGEUL_CHARSET;
           FontName := 'Batang';
          End;
  //Chinese_PRC: Use Default;
  End;
 End;

Function TMomoko.GetCountry:TCountry;
 Var
  s: Word;
  Country: TCountry;
 Begin
  s := GetSystemDefaultLangID;
  Country := UnKnown;
  Case s Of
   $0001: Country := Arabic;
   $0004: Country := Chinese;
   $0009: Country := English;
   $0401: Country := Saudi_Arabia;
   $0402: Country := Bulgarian;
   $0403: Country := Catalan;
   $0404: Country := Chinese_Taiwan;
   $0405: Country := Czech;
   $0406: Country := Danish;
   $0407: Country := German;
   $0408: Country := Greek;
   $0409: Country := United_States;
   $040A: Country := Traditional_Sort;
   $040B: Country := Finnish;
   $040C: Country := French;
   $040D: Country := Hebrew;
   $040E: Country := Hungarian;
   $040F: Country := Icelandic;
   $0410: Country := Italian;
   $0411: Country := Japanese;
   $0412: Country := Korean;
   $0413: Country := Dutch;
   $0414: Country := Norwegian;
   $0415: Country := Polish;
   $0416: Country := Brazil;
   $0417: Country := Rhaeto_Romanic;
   $0418: Country := Romanian;
   $0419: Country := Russian;
   $041A: Country := Croatian;
   $041B: Country := Slovak;
   $041C: Country := Albanian;
   $041D: Country := Swedish;
   $041E: Country := Thai;
   $041F: Country := Turkish;
   $0420: Country := Urdu;
   $0421: Country := Indonesian;
   $0422: Country := Ukrainian;
   $0423: Country := Belarusian;
   $0424: Country := Slovenian;
   $0425: Country := Estonian;
   $0426: Country := Latvian;
   $0427: Country := Lithuanian;
   $0429: Country := Farsi;
   $042A: Country := Vietnamese;
   $042D: Country := Basque;
   $042E: Country := Serbian;
   $042F: Country := FYROM;
   $0430: Country := Sutu;
   $0431: Country := Tsonga;
   $0432: Country := Tswana;
   $0434: Country := Xhosa;
   $0435: Country := Zulu;
   $0436: Country := Afrikaans;
   $0438: Country := Faeroese;
   $0439: Country := Hindi;
   $043A: Country := Maltese;
   $043C: Country := Gaelic;
   $043D: Country := Yiddish;
   $043E: Country := Malaysia;
   $0801: Country := Iraq;
   $0804: Country := Chinese_PRC;
   $0807: Country := Switzerland;
   $0809: Country := United_Kingdom;
   $080A: Country := Spanish_Mexico;
   $080C: Country := French_Belgium;
   $0810: Country := Italian_Switzerland;
   $0813: Country := Dutch_Belgium;
   $0814: Country := Norwegian_Nynorsk;
   $0816: Country := Portuguese_Portugal;
   $0818: Country := Romanian_Moldova;
   $0819: Country := Russian_Moldova;
   $081A: Country := Serbian_Latin;
   $081D: Country := Swedish_Finland;
   $0C01: Country := Arabic_Egypt;
   $0C04: Country := Chinese_Hong_Kong_SAR;
   $0C07: Country := German_Austria;
   $0C09: Country := English_Australia;
   $0C0A: Country := Spanish_International_Sort;
   $0C0C: Country := French_Canada;
   $0C1A: Country := Serbian_Cyrillic;
   $1001: Country := Arabic_Libya;
   $1004: Country := Chinese_Singapore;
   $1007: Country := German_Luxembourg;
   $1009: Country := English_Canada;
   $100A: Country := Spanish_Guatemala;
   $100C: Country := French_Switzerland;
   $1401: Country := Arabic_Algeria;
   $1407: Country := German_Liechtenstein;
   $1409: Country := English_New_Zealand;
   $140A: Country := Spanish_Costa_Rica;
   $140C: Country := French_Luxembourg;
   $1801: Country := Arabic_Morocco;
   $1809: Country := English_Ireland;
   $180A: Country := Spanish_Panama;
   $1C01: Country := Arabic_Tunisia;
   $1C09: Country := English_South_Africa;
   $1C0A: Country := Spanish_Dominican_Republic;
   $2001: Country := Arabic_Oman;
   $2009: Country := English_Jamaica;
   $200A: Country := Spanish_Venezuela;
   $2401: Country := Arabic_Yemen;
   $240A: Country := Spanish_Colombia;
   $2801: Country := Arabic_Syria;
   $2809: Country := English_Belize;
   $280A: Country := Spanish_Peru;
   $2C01: Country := Arabic_Jordan;
   $2C09: Country := English_Trinidad;
   $2C0A: Country := Spanish_Argentina;
   $3001: Country := Arabic_Lebanon;
   $300A: Country := Spanish_Ecuador;
   $3401: Country := Arabic_Kuwait;
   $340A: Country := Spanish_Chile;
   $3801: Country := Arabic_U_A_E;
   $380A: Country := Spanish_Uruguay;
   $3C01: Country := Arabic_Bahrain;
   $3C0A: Country := Spanish_Paraguay;
   $4001: Country := Arabic_Qatar;
   $400A: Country := Spanish_Bolivia;
   $440A: Country := Spanish_El_Salvador;
   $480A: Country := Spanish_Honduras;
   $4C0A: Country := Spanish_Nicaragua;
   $500A: Country := Spanish_Puerto_Rico;
  End;
  GetCountry := Country;
 End;

Procedure TMomoko.SetCountry(Value: TCountry);
 Begin
 End;

Function TMomoko.GetVolume: Byte;
 Var
  Vol: TVolumeRec;
 Begin
  Vol.LongVolume := 0;
  If FDevices <> -1 Then waveOutGetVolume(FDevices,@Vol.LongVolume);
  Result := (Vol.LeftVolume + Vol.RightVolume) shr 9;
 End;

Procedure TMomoko.SetTransMode(Mode: TTransMode);
 Begin
  If (csDesigning In ComponentState) Then
   Begin
    Case Mode Of
    tmNoTrans: Begin
                FPicture.Handle := LoadBitmap(hInstance, 'DEFAULT');
               End;
    tmShiftJIS: Begin
                 FPicture.Handle := LoadBitmap(hInstance, 'JIS');
                End;
    tmBIG5: Begin
             FPicture.Handle := LoadBitmap(hInstance, 'BIG5');
            End;
    tmGB18030: Begin
                FPicture.Handle := LoadBitmap(hInstance, 'GB18030');
               End;
    End;
    Canvas.Draw(0, 0, FPicture);
   End;
  FTransMode := Mode;
 End;

Procedure TMomoko.SetVolume(aVolume: Byte);
 Var
  Vol: TVolumeRec;
 Begin
  If FDevices <> -1 Then
   Begin
    Vol.LeftVolume := aVolume Shl 8;
    Vol.RightVolume := Vol.LeftVolume;
    waveOutSetVolume(FDevices, Vol.LongVolume);
   End;
 End;

Procedure TMomoko.Notification(AComponent: TComponent; AOperation: TOperation);
 Begin
  Inherited Notification(AComponent, AOperation);
  If (AOperation = opRemove) Then
   Begin
   End;
 End;

Procedure TMomoko.WMPaint(Var Message: TWMPaint);
 Begin
   Inherited;
   If (csDesigning In ComponentState) Then Canvas.Draw(0, 0, FPicture);
   Message.Result := 0;
 End;

Procedure TMomoko.WMSize(Var Message: TWMSize);
 Begin
  Inherited;
  If (csDesigning In ComponentState) Then
   Begin
    If Width <> FPicture.Width Then Width := FPicture.Width;
    If Height <> FPicture.Height Then Height := FPicture.Height;
   End;
  Message.Result := 0;
 End;

Constructor TMomoko.Create(AOwner: TComponent);
 Begin
  Inherited Create(AOwner);
  Height := 28;
  Width := 28;
  InitVolume;
  If (csDesigning In ComponentState) Then
   Begin
    FPicture := TBitmap.Create;
    FPicture.Handle := LoadBitmap(hInstance, 'DEFAULT');
    Width := FPicture.Width;
    Height := FPicture.Height;
    SetZOrder(True);
   End
  Else
   Begin
    Width := 0;
    Height := 0;
    SetZOrder(False);
    Visible := False;
   End;
  TryStartDrehtInterpret; 
 End;

Destructor TMomoko.Destroy;
 Begin
  If (csDesigning In ComponentState) Then FPicture.Free;
  Inherited Destroy;
 End;

Procedure TMomoko.InitVolume;
 Var
  WaveOutCaps: TWaveOutCaps;
  i: Integer;
 Begin
  FDevices := -1;
  For i := 0 to waveOutGetNumDevs - 1 Do
   Begin
    waveOutGetDevCaps(I, @WaveOutCaps, SizeOf(WaveOutCaps));
    If (WaveOutCaps.dwSupport And WAVECAPS_VOLUME) <> 0 Then
     Begin
      FDevices := i;
      Break;
     End;
   End;
 End;

Function TMomoko.Translate(SentencesOfGB2312: String): String;
 Var
  s: WideString;
  p: AnsiString;
 Begin
  p := '';
  s := SentencesOfGB2312;
  Case FTransMode Of
  tmNoTrans: Translate := SentencesOfGB2312;
  tmShiftJIS: Translate := TranslateGBK2JISbyString(SentencesOfGB2312);
  tmBIG5: Translate := TranslateGBK2BIG5byString(SentencesOfGB2312);
  tmGB18030: Begin
              p := TranslateUnicodeStringGB18030(s);
              Translate := p;
             End; 
  End;
 End;

End.
